package net.sf.alchim.codeplus.spoonchecker;

import java.lang.reflect.Method;

import spoon.processing.AbstractProcessor;
import spoon.processing.ProblemFixer;
import spoon.processing.Property;
import spoon.processing.Severity;
import spoon.reflect.declaration.CtElement;

public abstract class AbstractAnalyzer<E extends CtElement> extends AbstractProcessor<E> {

    @Property
    public Severity severity = Severity.WARNING;

    @SuppressWarnings("unchecked")
    public AbstractAnalyzer() {
        super();
        clearProcessedElementType();
        boolean isEmpty = true;
        for (Method m : getClass().getMethods()) {
            if (m.getName().equals("analyze")
                    && (m.getParameterTypes().length == 1)) {
                Class c = m.getParameterTypes()[0];
                if (CtElement.class != c) {
                    addProcessedElementType(c);
                    isEmpty = false;
                }
            }
        }
        if (isEmpty) {
            addProcessedElementType(CtElement.class);
        }
    }

    final public void process(E element) {
        if ((severity.compareTo(Severity.WARNING) > 0) || !existsSuppressWarning(element)) {
            analyze(element);
        }
    }

    abstract public void analyze(E element);

    protected boolean existsSuppressWarning(CtElement element) {
        boolean back = false;
        if (element != null) {
            SuppressWarnings sw = element.getAnnotation(SuppressWarnings.class);
            if (sw != null) {
                for(String value :sw.value()) {
                    back = value.equals(this.getClass().getName());
                    back = back || value.equals(this.getClass().getSimpleName());
                    back = back || value.equals(getCategory());
                    if (back) {
                        break;
                    }
                }
            }
            if (!back) {
                back = existsSuppressWarning(element.getParent());
            }
        }
        return back;
    }

    protected String getCategory() {
        return "none";
    }

    @SuppressWarnings("unchecked")
    protected void report(CtElement element, String msg, ProblemFixer<CtElement>... fixers) {
        ProblemFixer<CtElement>[] nfixers = new ProblemFixer[fixers.length+1];
        nfixers[nfixers.length-1] = new SuppressWarningFixer(this.getClass());
        getEnvironment().report(this, severity, element, msg, nfixers);
    }
}
