package net.sf.alchim.codeplus.spoonchecker;

import java.util.Arrays;
import java.util.regex.Pattern;

import spoon.processing.Property;
import spoon.reflect.declaration.CtField;
import spoon.reflect.declaration.ModifierKind;
import spoon.reflect.declaration.CtEnum;

public class FieldNameAnalyzer extends AbstractAnalyzer<CtField<?>> {

    @Property
    public String staticFinalPattern = "[A-Z0-9][A-Z0-9_]*";

    @Property
    public String publicPattern = "[a-z][a-zA-Z0-9]*";

    @Property
    public String protectedPattern = "[a-z][a-zA-Z0-9]*_";

    @Property
    public String privatePattern = "[a-z][a-zA-Z0-9]*_";

    @Property
    public String enumPattern = "[A-Z0-9][A-Z0-9_]*";

    public void analyze(CtField<?> m) {
        SuppressWarnings sw = m.getAnnotation(SuppressWarnings.class);
        if ((sw != null) && Arrays.asList(sw.value()).contains(this.getClass().getName().substring(this.getClass().getName().lastIndexOf('.')))) {
            return;
        }
        if (m.getParent() instanceof CtEnum) {
            check(m, enumPattern);
        } else if (m.getModifiers().contains(ModifierKind.STATIC) && m.getModifiers().contains(ModifierKind.FINAL)) {
            check(m, staticFinalPattern);
        } else if (m.getModifiers().contains(ModifierKind.PUBLIC)) {
            check(m, publicPattern);
        } else if (m.getModifiers().contains(ModifierKind.PRIVATE)) {
            check(m, privatePattern);
        } else {
            check(m, protectedPattern);
        }
    }

    private void check(CtField<?> m, String pattern) {
        if (!Pattern.matches(pattern, m.getSimpleName())) {
            report(m, "'" + m.getSimpleName()+"' doesn't match pattern '"+pattern +"'");
        }
    }
}

