package net.sf.alchim.codeplus.spoonprocessor;

import spoon.processing.AbstractProcessor;
import spoon.processing.Severity;
import spoon.reflect.code.CtBlock;
import spoon.reflect.code.CtExpression;
import spoon.reflect.declaration.CtAnnotation;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtConstructor;
import spoon.reflect.declaration.CtExecutable;
import spoon.reflect.declaration.CtMethod;
import spoon.reflect.declaration.CtParameter;
import spoon.reflect.declaration.ModifierKind;
import spoon.template.Substitution;

import net.sf.alchim.codeplus.annotation.Default;
import net.sf.alchim.codeplus.annotation.Nullable;
import net.sf.alchim.codeplus.annotation.ParamsNotNull;

public class NullParamProcessor extends AbstractProcessor<CtExecutable<?>> {

    public void process(CtExecutable<?> m) {
        try {
            try {
                if (!(m instanceof CtMethod) && !(m instanceof CtConstructor)) {
                    getEnvironment().report(this, Severity.WARNING, m, "the processor only accept method and constructor");
                    return;
                }
                if (m.getModifiers().contains(ModifierKind.ABSTRACT)) {
                    getEnvironment().report(this, Severity.WARNING, m, "the processor only work on contrect method (not on abstract))");
                    return;
                }
                if (!(m.getDeclaringType() instanceof CtClass)) {
                    getEnvironment().report(this, Severity.WARNING, m, "the processor only work into class (not into interface)");
                    return;
                }
                addDefaulting(m);
                addNullChecker(m);
            } finally {
                removeAnnotations(m);
            }
        } catch (Exception exc) {
            throw new ProcessingException(this, m, exc);
        }
    }

    // TODO: add defaulting to same type as param
    protected void addDefaulting(CtExecutable<?> m) {
        for (CtParameter<?> param : m.getParameters()) {
            Default annotation = param.getAnnotation(Default.class);
            if (annotation == null) {
                continue;
            }
            if ((annotation.value() == null) || (annotation.value().length() == 0)) {
                continue;
            }
            CtExpression<Object> defaultExpr = getFactory().Code().createCodeSnippetExpression(annotation.value());
            CtBlock<?> defaultBlock = Substitution.substituteMethodBody((CtClass<?>) m.getDeclaringType(), new NullParamTemplate(param, defaultExpr), "setDefault", getFactory().Type()
                    .createReference(Object.class));
            m.getBody().insertBegin(defaultBlock);
        }
    }

    protected void addNullChecker(CtExecutable<?> m) {
        CtAnnotation<?> annotation = ProcessorHelper.findAnnotation(m, ParamsNotNull.class);
        if (annotation == null) {
            return;
        }
        for (CtParameter<?> param : m.getParameters()) {
            if (param.getType().isPrimitive()) {
                continue;
            }
            if (param.getAnnotation(Default.class) != null) {
                ProcessorHelper.removeAnnotation(param, Default.class);
                continue;
            }
            if (param.getAnnotation(Nullable.class) != null) {
                ProcessorHelper.removeAnnotation(param, Nullable.class);
                continue;
            }
            CtBlock<?> checkBlock = Substitution
                    .substituteMethodBody((CtClass<?>) m.getDeclaringType(), new NullParamTemplate(param, null), "check", getFactory().Type().createReference(Object.class));
            // checkBlock.accept(new
            // SetPositionVisitor(annotation.getPosition()));
            ProcessorHelper.forcePositionOfEveryStmt(checkBlock, annotation.getPosition());
            m.getBody().insertBegin(checkBlock);
        }

    }

    protected void removeAnnotations(CtExecutable<?> m) {
        for (CtParameter<?> param : m.getParameters()) {
            ProcessorHelper.removeAnnotation(param, Default.class);
            ProcessorHelper.removeAnnotation(param, Nullable.class);
        }
        ProcessorHelper.removeAnnotation(m, ParamsNotNull.class);
    }

}
