package net.sf.alchim.codeplus.spoonprocessor;

import spoon.processing.AbstractProcessor;
import spoon.reflect.code.CtBlock;
import spoon.reflect.declaration.CtAnnotation;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtField;
import spoon.reflect.declaration.ModifierKind;
import spoon.reflect.reference.CtFieldReference;
import spoon.template.Substitution;

import net.sf.alchim.codeplus.annotation.GenToString;
import net.sf.alchim.codeplus.annotation.NotInToString;

public class ToStringGenerator extends AbstractProcessor<CtClass<?>> {

    public void process(CtClass<?> type) {
        try {
            CtAnnotation<?> annotation = ProcessorHelper.findAnnotation(type, GenToString.class);
            if (annotation == null) {
                return;
            }
            final StringBuilder ops = new StringBuilder();
            ops.append("str.append(\"").append(type.getQualifiedName()).append("[\")");
            if ((type.getSuperclass() != null) && (type.getSuperclass().getAllFields() != null)) {
                for (CtFieldReference<?> field : type.getSuperclass().getAllFields()) {
                    if ((field.getAnnotation(NotInToString.class) == null) && !field.getModifiers().contains(ModifierKind.PRIVATE)) {
                        ops.append(".append(\"").append(field.getSimpleName()).append(":\").append(").append(field.getSimpleName()).append(").append(',')");
                    }
                }
            }
            if (type.getFields() != null) {
                for (CtField<?> field : type.getFields()) {
                    if (field.getAnnotation(NotInToString.class) == null) {
                        ops.append(".append(\"").append(field.getSimpleName()).append(":\").append(").append(field.getSimpleName()).append(").append(',')");
                    } else {
                        ProcessorHelper.removeAnnotation(field, NotInToString.class);
                    }
                }
            }
            ops.append(".append(']');");
            CtBlock<?> block = getFactory().Core().createBlock();
            block.getStatements().add(getFactory().Code().createCodeSnippetStatement(ops.toString()));
            Substitution.insertAllMethods(type, new ToStringTemplate(block));

            ProcessorHelper.removeAnnotation(type, GenToString.class);
        } catch (Exception exc) {
            throw new ProcessingException(this, type, exc);
        }
    }

}
